#!/bin/bash

#Do regress tests for the zFilter switch implementation
#First define interfaces that can be used for testing
#If reveiving of pacet fails, namely "receive 500 pacets"
#make sure that tcpdump is available

IFACE1=eth0 #interface that is connected to port0
IFACE2=eth2 #interface that is connected to port1

# Helper funtions to check the results 
function compare_result {
    if [[ "$1" = "$6" && "$2" = "$7" && "$3" = "$8" && "$4" = "$9" ]]; then
	echo PASSED
    else
	echo FAILED
    fi
}

function check_counters {
    
    if [[ "$1" = "$6" && "$2" = "$7" && "$3" = "$8" && "$4" = "$9" ]]; then
	echo PASSED
    else
	echo FAILED
    fi
}

##### Test reading and writing IDs
echo Test writing and reading ids...
../sw/PubSubManager/psm set filter file ids1
echo Ids set

sleep 1s
../sw/PubSubManager/psm get filter file tmp

RESULT=`diff ids1 tmp`

rm tmp

if [ "$RESULT" = "" ]; then
    echo ID write and read test passed
else
    echo ID write and read test failed. Diff:$RESULT
    exit
fi

##### Test packet sending
FAILS=0

echo Test sending and receiving of the packets
echo sending to port1 from $IFACE1

FILTER=00000000000000000000000000000000000000000000000000000000000000F0
for i in `seq 16`
do
	TMP=${FILTER:($i)}"F"${FILTER:0:($i-1)}
	TMP=${FILTER:0:($i-1)}"F"${FILTER:($i)}
	RESULT=`../sw/PubSubManager/psm send packet ttl 1 tid $((($i-1)/4)) if $IFACE1 cnt 1 delay 100 len 1000 output $TMP`
	TMP=`compare_result 0 1 0 0 $RESULT`
	if [ "$TMP" = "PASSED" ]; then
	    echo -n id:$((($i-1)/4)), 
	else
	    echo send to port 3 from 0 with tid 1 FAILED
	    echo $RESULT
	    FAILS=$[FAILS+1]
	fi
done
echo
echo sending to port2 from $IFACE1

FILTER=0000000000000000000000000000000000000000000000000000000000000F00
for i in `seq 16`
do
	TMP=${FILTER:($i)}"F"${FILTER:0:($i-1)}
	TMP=${FILTER:0:($i-1)}"F"${FILTER:($i)}
	RESULT=`../sw/PubSubManager/psm send packet ttl 1 tid $((($i-1)/4)) if $IFACE1 cnt 1 delay 100 len 1000 output $TMP`
	   
	TMP=`compare_result 0 0 1 0 $RESULT`
	if [ "$TMP" = "PASSED" ]; then
	    echo -n id:$((($i-1)/4)), 
	else
	    echo send to port 3 from 0 with tid 1 FAILED
	    echo $RESULT
	    FAILS=$[FAILS+1]
	fi
done

echo
echo sending to port3 from $IFACE1

FILTER=000000000000000000000000000000000000000000000000000000000000F000
for i in `seq 16`
do
	TMP=${FILTER:($i)}"F"${FILTER:0:($i-1)}
	TMP=${FILTER:0:($i-1)}"F"${FILTER:($i)}
	RESULT=`../sw/PubSubManager/psm send packet ttl 1 tid $((($i-1)/4)) if $IFACE1 cnt 1 delay 100 len 1000 output $TMP`
	TMP=`compare_result 0 0 0 1 $RESULT`
	if [ "$TMP" = "PASSED" ]; then
	    echo -n id:$((($i-1)/4)), 
	else
	    echo send to port 3 from 0 with tid 1 FAILED
	    echo $RESULT
	    FAILS=$[FAILS+1]
	fi
done

echo
echo sending to port1 and port3 from $IFACE1

FILTER=000000000000000000000000000000000000000000000000000000000000F0F0
for i in `seq 16`
do
	TMP=${FILTER:($i)}"F"${FILTER:0:($i-1)}
	TMP=${FILTER:0:($i-1)}"F"${FILTER:($i)}
	RESULT=`../sw/PubSubManager/psm send packet ttl 1 tid $((($i-1)/4)) if $IFACE1 cnt 1 delay 100 len 1000 output $TMP`
	TMP=`compare_result 0 1 0 1 $RESULT`
	if [ "$TMP" = "PASSED" ]; then
	    echo -n id:$((($i-1)/4)), 
	else
	    echo send to port 3 from 0 with tid 1 FAILED
	    echo $RESULT
	    FAILS=$[FAILS+1]
	fi
done

echo

#Test that packets actually go trough by using tcpdump
echo Test sending of 500 packets from $IFACE1 to $IFACE2

/usr/sbin/tcpdump -i $IFACE2 -q -n 2>dump > /dev/null &
DUMP_PID=$!

sleep 1s

../sw/PubSubManager/psm send packet ttl 1 tid 0 if $IFACE1 delay 1 cnt 500 len 1000 output F00000000000000000000000000000000000000000000000000000000000F0F0 > /dev/null

sleep 1s
#kill %1
kill $DUMP_PID
sleep 1s

RESULT=`grep "packets captured" dump`

echo Received ${RESULT:0:3} packets from $IFACE2

if [ "${RESULT:0:3}" = "500" ]; then
	echo PASSED
else
	echo FAILED:
	cat dump
	FAILS=$[FAILS+1]
fi

rm dump

echo Test sending of 500 packets from $IFACE2 to $IFACE1

/usr/sbin/tcpdump -i $IFACE1 -q -n 2>dump > /dev/null &
DUMP_PID=$!

sleep 1s

../sw/PubSubManager/psm send packet ttl 1 tid 0 if $IFACE2 delay 1 cnt 500 len 1000 output F00000000000000000000000000000000000000000000000000000000000000F > /dev/null

sleep 1s
#kill %2
kill $DUMP_PID
sleep 1s

RESULT=`grep "packets captured" dump`

echo Received ${RESULT:0:3} packets from $IFACE1

if [ "${RESULT:0:3}" = "500" ]; then
	echo PASSED
else
	echo FAILED
	cat dump
	FAILS=$[FAILS+1]
fi

rm dump

# Test bit set bit counter

echo Try to send with full zFilter
FILTER=fffffffffffffffffffffffffffffffffffffffffffffffffff0ffffffffffff

RESULT=`../sw/PubSubManager/psm send packet ttl 1 tid $((($i-1)/4)) if $IFACE1 cnt 1 delay 100 len 1000 output $FILTER`
TMP=`compare_result 0 0 0 0 $RESULT`
if [ "$TMP" = "PASSED" ]; then
    echo Passed, packet dropped.
else
    echo send to port 3 from 0 with tid 1 FAILED
    echo $RESULT
    FAILS=$[FAILS+1]
fi

echo Test setting of set bit limit to 1 and then send

regwrite 0x02103004 0x1 > /dev/null

FILTER=F00000000000000000000000000000000000000000000000000000000000F000

RESULT=`../sw/PubSubManager/psm send packet ttl 1 tid $((($i-1)/4)) if $IFACE1 cnt 1 delay 100 len 1000 output $FILTER`
TMP=`compare_result 0 0 0 0 $RESULT`
if [ "$TMP" = "PASSED" ]; then
    echo Passed, packet dropped.
else
    echo send to port 3 from 0 with tid 1 FAILED
    echo $RESULT
    FAILS=$[FAILS+1]
fi

regwrite 0x02103004 0x28 > /dev/null

echo Send with ttl 0

FILTER=F00000000000000000000000000000000000000000000000000000000000F000

RESULT=`../sw/PubSubManager/psm send packet ttl 0 tid $((($i-1)/4)) if $IFACE1 cnt 1 delay 100 len 1000 output $FILTER`
TMP=`compare_result 0 0 0 0 $RESULT`
if [ "$TMP" = "PASSED" ]; then
    echo Passed, packet dropped.
else
    echo send to port 3 from 0 with tid 1 FAILED
    echo $RESULT
    FAILS=$[FAILS+1]
fi

echo Send with ttl 0x19 and check ttl of incoming packet 

/usr/sbin/tcpdump -i $IFACE1 >dump 2>/dev/null &
DUMP_PID=$!
sleep 1s

../sw/PubSubManager/psm send packet ttl 25 tid 0 if $IFACE2 delay 1 cnt 1 len 1000 output F00000000000000000000000000000000000000000000000000000000000000F > /dev/null

sleep 1s
#echo $DUMP_PID
kill $DUMP_PID
sleep 1s

RESULT=`grep "0x0000:" dump`

echo Received packet, with TTL ${RESULT:15:2} from $IFACE1

if [ "${RESULT:15:2}" = "18" ]; then
	echo PASSED
else
	echo FAILED, received packet with header:
	cat dump
	FAILS=$[FAILS+1]
fi

rm dump

echo Failed tests: $FAILS
