/*
* Copyright (c) 2009, Nomadiclab, Oy LM Ericsson Ab <pubsub@nomadiclab.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* Alternatively, this software may be distributed under the terms of BSD
* license.
* 
* See LICENSE and COPYING for more details.
*/

module bit_count
  #(parameter WIDTH = 64)
    (input wire [WIDTH-1:0] in,
     output wire [7:0] out); 
   
   function integer log2;
      input integer number;
      begin
         log2=0;
         while(2**log2<number) begin
            log2=log2+1;
         end
      end
   endfunction // log2
   
   localparam MAX_STAGES = log2(WIDTH)-1;
   
   wire [WIDTH-1:0] stages[MAX_STAGES:0];
   
   genvar 	    i,j;
   
   generate
      for(i=0;i<WIDTH;i=i+2) begin: stage0
	 assign stages[0][i+1] = in[i] & in[i+1];
	 assign stages[0][i] = in[i] ^ in[i+1];
      end
   endgenerate
   
   generate
      for(i=1;i<MAX_STAGES+1;i=i+1) begin: rows 
	 for(j=0;j<WIDTH;j=j+2**(i+1)) begin: columns
	    assign stages[i][j+i+1 : j] = stages[i-1][j+i : j] + stages[i-1][j + i + ((2**(i+1))/2) : j + ((2**(i+1))/2)];
	 end
      end
   endgenerate
   
   assign 	   out = stages[MAX_STAGES][7:0];
   
endmodule
