/*
* Copyright (c) 2009, Nomadiclab, Oy LM Ericsson Ab <pubsub@nomadiclab.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* Alternatively, this software may be distributed under the terms of BSD
* license.
* 
* See LICENSE and COPYING for more details.
*/

module id_store 
  #(parameter DATA_WIDTH = 64,
    parameter DEPTH = 1,
    parameter INDEX_LENGTH= 1)
    (
     
     input				read,
     input [INDEX_LENGTH-1:0]		index,
     output reg	[DATA_WIDTH-1:0]	data,				
     
     //access for register operations
     input				write_reg,
     input				read_reg,
     input [INDEX_LENGTH-1:0]		index_reg,
     input				index_reg_hig,
     input [DATA_WIDTH/2-1:0]		data_reg_in,
     output reg	[DATA_WIDTH/2-1:0]	data_reg_out,
     output reg				data_reg_rdy,				
     
     input				clk
     ); 
   
   reg [DATA_WIDTH/2-1:0]		RAM_HIGH [DEPTH-1:0];
   reg [DATA_WIDTH/2-1:0]		RAM_LOW [DEPTH-1:0];
   
   wire					reg_ena;
   
   assign				reg_ena = read_reg | write_reg;
   
   always @(posedge clk) begin
      data_reg_rdy <= 0;
      if(reg_ena) begin
	 if(write_reg) 
	   if(index_reg_hig) begin
	      RAM_HIGH[index_reg] <= data_reg_in;
	   end else
	     RAM_LOW[index_reg] <= data_reg_in;
	 else 
	   if(index_reg_hig) begin
	      data_reg_out <= RAM_HIGH[index_reg];
	   end else
	     data_reg_out <= RAM_LOW[index_reg];
	 data_reg_rdy <= 1;
      end
   end
   
   always @(posedge clk) begin
      if(read) begin
	 data[31:0] 	<= RAM_LOW[index];
	 data[63:32] <= RAM_HIGH[index];
      end
   end
   
endmodule
