#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <stdint.h>
#include <unistd.h>

#include "configparser.h"
#include "psm.h"
#include "helpers.h"

extern struct global_args g;
extern struct psm_conf *yyconf;
extern struct psm_conf psmconf;
extern int yyparse();

#define PARSE_STRING 1
#define PARSE_FILE 2

int getinput(char *buf, int maxlen)
{
        int retval = 0;

        if (maxlen > g.parser.toparse)
                maxlen = g.parser.toparse;

        if (g.parser.mode == PARSE_FILE) {
                if (!g.parser.file) {
                        dprint(LOG_ERR, "Broken pipe\n");
                        return 0;
                }
                retval = fread(buf, 1, maxlen, g.parser.file);
        } else {
                memcpy(buf, g.parser.buffer+g.parser.offset, maxlen);
                g.parser.offset += maxlen;
                g.parser.toparse -= maxlen;
                retval = maxlen;
        }
        return retval;
}

int parse_mem(char *str, int len, struct psm_conf *psm)
{
        g.parser.buffer = str;
        g.parser.toparse = len;
        g.parser.mode = PARSE_STRING;
        yyconf = psm;
        return yyparse();
}

/* multiple commands are not implemented in language syntax yet */
int parse_file(FILE *f)
{
        g.parser.mode = PARSE_FILE;
        g.parser.file = f;
        yyconf = &psmconf;
        return yyparse();
}

int parse_args_opts(int argc, char **argv)
{
        char c;
        int i;

        i = 1;
	while ((c = getopt (argc, argv, "vshd:")) != -1) {
		switch (c) {
                case 'v':	/* Verbose */
                        g.opt.verbose += 1;
                        i++;
			break;
                case 's':
                        g.opt.simulate = 1;
                        i++;
                        break;
                case 'd':
                        g.opt.nf2_dev = strdup(optarg);
                        i+=2;
                        break;
                case 'h':
                default:
                        usage();
		}
	}

        return i;
}

int parse_args(int argc, char **argv, int off, struct psm_conf *psm)
{
        char cmdbuffer[1026];  /* let's try to have a buffer overflow */
        int remain = 1024;
        int i;

        memset(cmdbuffer, 0, 1024);

        /* isn't stack guaranteed to be zeroed? */
        for(;off<argc;off++) {
                strncat(cmdbuffer, argv[off], remain);
                remain -= (strlen(argv[off]) + 1);
                cmdbuffer[strlen(cmdbuffer)] = ' ';
        }

        /* call our language parser */
        i = parse_mem(cmdbuffer, strlen(cmdbuffer), psm);
        return i;
}
