#include <string.h>
#include <stdint.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "configparser.h"
#include "psm.h"
#include "helpers.h"

extern struct global_args g;

/* CONFIG file format:  Tid:Port:Link:<64 characters (hex)> */


void cp_free(ztable table)
{
        int i;

        for(i=0;i<g.nfconf.num_tids*g.nfconf.num_ports*g.nfconf.num_links;i++) 
        {
                if (table[i])
                        free(table[i]);
        }
        free(table);
}

zfilter
zf_str_to_hex(const char *str, zfilter res)
{
    zfilter tmp;
    int i;
    char mystr[3] = {0};

    if (res)
            tmp = res;
    else {
            tmp = malloc(ZFILTER_SIZE);
            if (!tmp)
                    return NULL;
    }

    for(i=0;i<ZFILTER_SIZE*2;i+=2) {
            mystr[0] = *(str+i);
            mystr[1] = *(str+i+1);
            tmp[i/2] = (uint8_t)(strtol(mystr, NULL, 16) & 0xFF);
    }

    return tmp;
#if 0
    for(i=0;i<LID_STR_LEN;i+=2) {
            uint8_t byte,t;
            char a,b;

            if (!isxdigit(str[i]) || !isxdigit(str[i+1])) {
                    dprint(LOG_ERR, "Illegal character in filter: %c %c\n", 
                           str[i], 
                           str[i+1]);
                    free(tmp);
                    return NULL;
            }
            a = toupper(str[i]);
            b = toupper(str[i+1]);
            
            t = (a - '0');
            byte = (t > 9 ? 10 + (a - 'A') : t) << 4;
            t = (b - '0');
            byte |= (t > 9 ? 10 + (b - 'A') : t);
            
            tmp[i/2] = byte;
    }
    return tmp;
#endif
}

ztable
cp_parse(char *buffer)
{
        ztable table;
        char delim[3] = ":\n";
        char *token;

        delim[2] = 0;

        table = calloc(g.nfconf.num_tids*g.nfconf.num_ports*g.nfconf.num_links,
                       sizeof(uint8_t *));

        token = strtok(buffer, delim);
        if (!token) {
                dprint(LOG_ERR, "Parsing failed\n");
                free(table);
                return NULL;
        }

        do {
                int tid;
                int port;
                int link;
                char *filt;

                tid = atoi(token);
                if (tid < 0 || tid >= g.nfconf.num_tids) {
                        dprint(LOG_ERR, "Illegal TID (%d)\n", tid);
                        cp_free(table);
                        return NULL;
                }

                token = strtok(NULL, delim);
                if (!token) {
                        dprint(LOG_ERR, "Parsing PORT failed\n");
                        cp_free(table);
                        return NULL;
                }

                port = atoi(token);
                if (port < 0 || port >= g.nfconf.num_ports) {
                        dprint(LOG_ERR, "Illegal Port (%d)\n", port);
                        cp_free(table);
                        return NULL;
                }

                token = strtok(NULL, delim);
                if (!token) {
                        dprint(LOG_ERR, "Parsing LID failed\n");
                        cp_free(table);
                        return NULL;
                }
        
                link = atoi(token);
                if (link < 0 || link >= g.nfconf.num_links) {
                        dprint(LOG_ERR, "Illegal LINK (%d)\n", link);
                        cp_free(table);
                        return NULL;
                }
        
                token = strtok(NULL, delim);
                if (!token) {
                        dprint(LOG_ERR, "Parsing failed\n");
                        cp_free(table);
                        return NULL;
                }
        
                filt = token;
                if (strlen(filt) < 64) {
                        dprint(LOG_ERR, "Too short filter (%d bytes)\n", 
                               (int)strlen(filt));
                        cp_free(table);
                        return NULL;
                }
                if (strlen(filt) > 64) {
                        dprint(LOG_ERR, "Too large filter (%d bytes)\n", 
                               (int)strlen(filt));
                        cp_free(table);
                        return NULL;
                }

        
                if (GETFILTER(table, tid, port, link)) {
                        dprint(LOG_INF, "Replacing pre-configured filter on "
                               "tid: %d, port: %d lid: %d\n", tid, port, link);
                        free(GETFILTER(table, tid, port, link));
                }

                GETFILTER(table, tid, port, link) = zf_str_to_hex(filt, NULL);
                if (!GETFILTER(table, tid, port, link)) {
                        dprint(LOG_ERR, "Converting to hex failed for tid: %d "
                               "port: %d, lid: %d\n", tid, port, link);
                        cp_free(table);
                        return NULL;
               }
                
        } while( (token = strtok(NULL, delim)) );

        return table;
}

#if 0
zfilter 
cp_read_and_parse(const char *filename)
{
        char *buffer;
        FILE *f = NULL;
        zfilter fp;

        buffer = malloc(MAX_FILTER_FILESIZE);
        if (!buffer)
                return NULL;
        
        memset(buffer, 0, MAX_FILTER_FILESIZE);

        f = fopen(filename, "r");
        if (!f) {
                free(buffer);
                return NULL;
        }

        if (fread(buffer, 1, MAX_FILTER_FILESIZE, f) == MAX_FILTER_FILESIZE) {
                printf("Config file too big\n");
                
                free(buffer);
                fclose(f);
                return NULL;
        }

        cp_parse(buffer);
        
        fclose(f);
        free(buffer);
        return fp;
}
#endif

#if STANDALONE
int main(int argc, char **argv)
{
    port fp;
    int i;
    int j,z;
    uint8_t *data;

    fp = cp_read_and_parse(argv[1]);
    if (!fp) {
        printf("Error while parsing %s\n", argv[1]);
        return -1;
    }

    printf("fp=%p, fp[0] = %p, fp[0][0] = %p\n", fp, fp[0], fp[0][0]);

    for(i=0;i<NUMPORTS;i++) {
        for(j=0;j<NUMLIDS;j++) {
            printf("[%d,%d] = ", i, j);
            data = fp[i][j];
            if (data == NULL)
                printf("(null)");
            else
                for(z=0;z<LID_STR_LEN/2;z++)
                    printf("%02x", data[z]);
            printf("\n");
        }
    }

    cp_free(fp);
    return 1;
}
#endif
