#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <stdint.h>
#include <stdarg.h>

#include "nflib/api.h"
#include "common/zf_defines.h"
#include "configparser.h"
#include "psm.h"
#include "cmds.h"
#include "helpers.h"

extern struct global_args g;
extern struct psm_conf psmconf;

void add_to_nf2list(struct nf2_cmd **pprev, struct nf2_cmd *this)
{
        struct nf2_cmd *prev = *pprev;

        dprint(LOG_DBG, "Adding command:\n");
        dprint(LOG_DBG, "    cmd = %x, addr = 0x%x, len = %d, target = %p\n",
               this->cmd, this->addr, this->len, this->target);

        if (prev) {
                prev->next = this;
                this->first = prev->first;
        } else {
                this->first = this;
                if (!g.cleanup.cmdchain)
                        g.cleanup.cmdchain = this;
        }
        *pprev = this;
}

/* based on spesifications, determine the hw address */
int calc_nf2_address(struct psm_conf *spex)
{
        int addr = ZF_FILTER_BASE;
        int pl = 0;

        /* set tid */
        if (spex->tid != INDEX_ANY)
                addr |= (((spex->tid << ZF_FILTER_TID_OFF) & 
                          g.nfconf.tid_mask));

        /* set port & link */
        if (spex->port != INDEX_ANY)
                pl = spex->port * g.nfconf.num_links;
        
        if (spex->link != INDEX_ANY)
                pl += spex->link;

        addr |= ((pl << g.nfconf.pl_off) & g.nfconf.pl_mask);


        return addr;
}

/* Based on the address deduce port, link and table id (tid) */
uint32_t decode_nf2_address(int addr)
{
        int port, tid, link;
        int pl;

        tid = ZF_FILTER_TID(addr);
        pl = ZF_FILTER_PL(addr);

        port = pl / g.nfconf.num_links;
        link = pl % g.nfconf.num_links;

        /* consistency check */
        if (!IN_VALID_TID_RANGE(tid) ||
            !IN_VALID_PORT_RANGE(port) ||
            !IN_VALID_LINK_RANGE(link)) {
                dprint(LOG_ERR, "Consistency error (decode_nf2_address): "
                       "0x%x, 0x%x, 0x%x!\n", tid, port, link);
                return 0;
        }

        return (tid << 24 | port << 16 | link << 8);
}

/* helper for creating bitmask */
int pick_mask(int len)
{
        /* simple naive implementation */
        int table[] = {0x00, 
                       0x01, 0x03, 0x07, 0x0F, 
                       0x1F, 0x3F, 0x7F, 0xFF};

        if (len > 8 || len < 0) {
                dprint(LOG_ERR, "pick_mask(): error!!!\n");
                abort();
        }

        return table[len];
}


/* make this a typed cleanup stack instead of global variables */
void global_exit()
{
        if (g.cleanup.libnet)
                libnet_destroy(g.cleanup.libnet);
        if (g.cleanup.extrabuf1)
                free(g.cleanup.extrabuf1);
        if (g.cleanup.extrabuf2)
                free(g.cleanup.extrabuf2);
        if (g.cleanup.packet)
                free(g.cleanup.packet);
        if (g.cleanup.iface)
                free(g.cleanup.iface);
        if (g.rttable)
                cp_free(g.rttable);
        if (g.cleanup.cmdchain)
                freecmd(g.cleanup.cmdchain);
        if (g.opt.nf2_dev)
                free(g.opt.nf2_dev);
        if (g.nf2)
                nflib_close(g.nf2);
}

void global_error_exit(char *err)
{
        printf("%s\n", err);
        global_exit();
        exit(-1);
}

void dprint(int lvl, char *fmt, ...)
{
        va_list ap;

        if (lvl <= g.opt.verbose) {
                va_start(ap, fmt);
                vprintf(fmt, ap);
                va_end(ap);
        }
}


/* convert binary zfilter into hexadecimal ascii format */
void gen_string(char *target, uint8_t *source)
{
        int i, j;

        for(i=0, j=0;i<ZFILTER_SIZE;i++,j+=2)
                snprintf(target+j, 3, "%02hhX", (int)source[i]);
}

