%option yylineno
%{
#include <string.h>
#include "parser.h"

static int lexer_getcmd(char *text);
static struct tok *lexer_get(char *find);
extern int getinput(char *, int);
#undef YY_INPUT
#define YY_INPUT(buf, retval, maxlen)    (retval = getinput(buf, maxlen))
%}

LINKSTR    "link"
FILESTR    "file"
OUTSTR     "output"
IFSTR      "if"

%x FILEMODE
%x OUTMODE
%x IFMODE
%%

<INITIAL>{FILESTR}               { BEGIN FILEMODE; return FIL; }
<INITIAL>{OUTSTR}                { BEGIN OUTMODE; return OUTPUT; }
<INITIAL>{IFSTR}                { BEGIN IFMODE; return IF; }

<IFMODE>[[:alnum:]]+             { BEGIN 0; yylval.s = strdup(yytext);
                                    return STRING; }

<FILEMODE>[[:alnum:][:punct:]]+  { BEGIN 0; yylval.s = strdup(yytext); 
                                   return FILENAME; }

<OUTMODE>[[:digit:]]{1,2}":"[[:digit:]]{1,2}":"[[:digit:]]{1,2}  { BEGIN 0;
                                   yylval.s = strdup(yytext);
                                   return ZFSPEX; }

<OUTMODE>[[:xdigit:]]{64}         { BEGIN 0; yylval.s = strdup(yytext);
                                   return ZFILTER; }


<INITIAL>[[:alpha:]]+            { return lexer_getcmd(yytext); }
<INITIAL>[[:digit:]]+            { yylval.ival = atoi(yytext); 
                                   return NUMBER; } 
<INITIAL>[[:xdigit:]]+           { yylval.l = strtol(yytext, NULL, 16);
                                   return XNUMBER; }

<FILEMODE,OUTMODE,IFMODE><<EOF>>        { BEGIN 0; return LEXERR; }

<INITIAL,FILEMODE,OUTMODE>[ ]+           /* eat space */
%%

#include <string.h>

#define T(m) { #m, m }

static struct tok {
    char *name;
    int tag;
} tokens[] = {
    T(GET),
    T(SET),
    T(CLEAR),
    T(SEND),
    T(FILTER),
    T(STATS),
    T(PACKET),
    T(PORT),
    T(TID),
    T(LINK),
    T(IFACE),
    T(SIZE),
    T(TTL),
    T(FIL),
    T(CNT),
    T(DELAY),
    T(LEN),
    T(PROTO)
};

#define TOKEN_COUNT (sizeof(tokens)/sizeof(tokens[0]))

static struct tok *lexer_get(char *find)
{
  int i;
  
    for(i=0;i<TOKEN_COUNT;i++)
        if (strcasecmp(find, tokens[i].name) == 0)
            return &tokens[i];
        
    return NULL;
}

static int lexer_getcmd(char *text)
{
    struct tok *token;

    token = lexer_get(text);
    if (token == NULL)
        return LEXERR;

    return token->tag;
}



