%{
#include <stdlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <stdint.h>

#include "configparser.h"
#include "psm.h"

  /* LANGUAGE:
   *
   * cmd := fcmd   [FILTER] fspex
   *      | fcmd    [STATS] fspex
   *      | [SEND] [PACKET] pspex
   *
   * fcmd := GET
   *       | SET
   *
   * fspex := [TID]  [NUMBER] fspex
   *        | [PORT] [NUMBER] fspex
   *        | [LINK] [NUMBER] fspex
   *        | [FILE] [STRING] fspex
   *        |
   *
   * pspex := [TTL]    [NUMBER] pspex
   *        | [IF]     [STRING] pspex
   *        | [CNT]    [NUMBER] pspex
   *        | [DELAY]  [NUMBER] pspex
   *        | [OUTPUT]  zfilter pspex
   *        | [LEN]    [NUMBER] pspex
   *        | [PROTO]  [NUMBER] pspex
   *        | [FILE]   [STRING] pspex
   *        | [TID]    [NUMBER] pspex
   * 
   * zfilter := [NUMBER:NUMBER:NUMBER]
   *          | [HEXNUMBER*64]
   *
   */

extern int yylineno;
extern char *yytext;
extern void yyerror(char *);
extern struct global_args g;
struct psm_conf *yyconf;

void yyerror(char *str)
{
        printf("Parser error: %s\n", str);
}

int set_val(char *str, int *target)
{
        int x;

        if (!str)
                return -1;

        x = atoi(str);
        if (x > 32) {
                yyerror("Illegal OUTPUT command");
                return -1;
        }

        *target = x;
        return 0;
}

%}


%Union {
  int ival;
  char *s;
  long int l;
}

%token    GET
%token    SET
%token    CLEAR
%token    SEND
%token    FILTER
%token    STATS
%token    PACKET
%token    PORT
%token    TID
%token    LINK
%token    IFACE
%token    SIZE
%token    TTL
%token    FIL
%token    IF
%token    CNT
%token    DELAY
%token    LEN
%token    PROTO
%token    OUTPUT
%token    LEXERR
%token    <s> ZFILTER
%token    <s> FILENAME
%token    <s> ZFSPEX
%token    <s> STRING
%token    <ival> NUMBER
%token    <l> XNUMBER
%%
statement: cmd
           {
                   if (GET_CMD(yyconf) == CMD_SET) {
                           if (!yyconf->file && !yyconf->prebuffer)
                                   yyconf->file = stdin;
                   } else if (GET_CMD(yyconf) == CMD_GET) {
                           if (!yyconf->file)
                                   yyconf->file = stdout;
                   } 
           }
         | lexerr 
         ;

cmd:       fcmd FILTER fspex
           {
                yyconf->command |= TGT_FILTER;
           }
         | fcmd STATS fspex
           {
                yyconf->command |= TGT_STATS;
           }
         | SEND PACKET pspex
           {
               yyconf->command = CMD_SEND | TGT_PACKET;
           }
         ;


fcmd:      GET
           {
               yyconf->command = CMD_GET;
           }
         | SET
           {
               yyconf->command = CMD_SET;

           }
         | CLEAR
           {
               yyconf->command = CMD_CLEAR;
           }
         ;


fspex:     fopt fspex
         |
         ;

fopt:	   port
         | tid
         | link
         | file
         ;



pspex:     popt pspex
         |
         ;

popt:      TTL NUMBER 
           {
                   if ($2 > 255 || $2 < 0) {
                           yyerror("TTL is limited to [0,255]");
                           return -1;
                   }
               yyconf->ttl = $2;
           }
         | IF STRING
           {
                   if (yyconf->iface) {
                           if (g.cleanup.iface == yyconf->iface)
                                   g.cleanup.iface = NULL;
                           free(yyconf->iface);
                   }
                   yyconf->iface = strdup($2);
                   g.cleanup.iface = yyconf->iface;
           }
         | CNT NUMBER
           {
                   if ($2 > 1000 || $2 < 1) {
                       yyerror("Count is limited to value between 1 and 1000");
                       return -1;
                   }
               yyconf->cnt = $2;
           }
         | DELAY NUMBER
           {
                   if ($2 > 1000*60 || $2 < 0) {
                           yyerror("Delay is limited to [0,60000]");
                           return -1;
                   }
               yyconf->delay = $2;
           }
         | OUTPUT ZFSPEX
           {
               /* do simple parsing: TID:PORT:LINK */
               char *tok;
               int err;

               tok = strtok($2, ":");
               err = set_val(tok, &yyconf->tid);
               tok = strtok(NULL, ":");
               err |= set_val(tok, &yyconf->port);
               tok = strtok(NULL, ":");
               err |= set_val(tok, &yyconf->link);

               if (err)
                       return err;

           }
         | OUTPUT ZFILTER
           {
                   strncpy((char *)yyconf->zfilter, $2, 64);
           }
         | LEN NUMBER
           {
                   if ($2 >= 1514) {
                           yyerror("Too long packet");
                           return -1;
                   }
                   yyconf->len = $2;
           }
         | PROTO XNUMBER
           {
               yyconf->proto = $2;
           }
         | file
         | tid
         ;

lexerr:    LEXERR
           {
               yyerror("Parser error\n");
               return -1;
           }
         ;

port:      PORT NUMBER
           {
                   if (IN_VALID_PORT_RANGE($2))
                           yyconf->port = $2;
                   else
                           yyerror("Invalid port");
           }
         ;

tid:       TID NUMBER
           {
                   if (IN_VALID_TID_RANGE($2))
                           yyconf->tid = $2;
                   else
                           yyerror("Invalid lit");
           }
         ;

link:      LINK NUMBER
           {
                   if (IN_VALID_LINK_RANGE($2))
                           yyconf->link = $2;
                   else
                           yyerror("Invalid link");
           }
         ;

file:      FIL FILENAME
           {
                   if (GET_CMD(yyconf) == CMD_SET ||
                       GET_CMD(yyconf) == CMD_SEND) {
                           yyconf->file = fopen($2, "r");
                           yyconf->prebuffer = malloc(MAX_FILTER_FILESIZE);
                           if (!yyconf->prebuffer) {
                                   yyerror("No memory for filter\n");
                                   exit(-1);
                           }
                           (void) fread(yyconf->prebuffer, 1, 
                                        MAX_FILTER_FILESIZE, yyconf->file);
                           fclose(yyconf->file);
                           yyconf->file = NULL;

                   } else if (GET_CMD(yyconf) == CMD_GET) {
                           yyconf->file = fopen($2, "w");
                   }
                   if (!yyconf->file && !yyconf->prebuffer) {
                           yyerror("Unable to open target file");
                   }
           }
         ;
