#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <stdint.h>
#include <math.h>
#include <netinet/ether.h>
#include <libnet.h>

#include "nflib/api.h"
#include "common/reg_defines.h"
#include "common/zf_defines.h"
#include "configparser.h"
#include "psm.h"
#include "cmds.h"
#include "helpers.h"

extern struct global_args g;

static void send_one_packet(libnet_t *lt, uint16_t proto, 
                            uint8_t *payload, int len)
{
        struct ether_addr SA, DA;
	char * err;

        ether_aton_r("FF:FF:FF:FF:FF:FF", &DA);
        ether_aton_r("00:12:34:56:78:9A", &SA);

	libnet_clear_packet(lt);

	if (libnet_build_ethernet((uint8_t *)&DA, (uint8_t *)&SA, proto, 
                                  payload, len, lt, 0) == -1)
	{
                err = libnet_geterror(lt);
		dprint(LOG_ERR, "Unable to build ethernet frame: %s\n", err);
                return;
	}

	libnet_write(lt);
}

static void create_payload(struct psm_conf *conf, uint8_t *payload)
{
        FILE *x;

        if (conf->prebuffer)
                memcpy(payload, conf->prebuffer, conf->len - 
                       sizeof(struct zfilter_fwd_header));
        else {

                x = fopen("/dev/urandom", "r");
                if (!x)
                        global_error_exit("No random seed");
                fread(payload, 1, conf->len - 
                      sizeof(struct zfilter_fwd_header), x);
                fclose(x);
                /* if read was short, we'll have to survive */
        }
}

void do_packet_send(struct psm_conf *conf)
{
        uint8_t *packet, *payload;
        struct zfilter_fwd_header *fw;
        libnet_t *lt; 
        uint8_t outzf[ZFILTER_SIZE];
        char libnet_err[LIBNET_ERRBUF_SIZE];
        struct psm_conf newcmd;
        int i, j;
        int *new, *old;

        packet = malloc(1500);
        if (!packet)
                global_error_exit("No memory");

        g.cleanup.packet = packet;

        fw = (struct zfilter_fwd_header *)packet;
        payload = packet + sizeof(*fw);

        /* let's see if we have source material for the packet */
        create_payload(conf, payload);


        lt = libnet_init(LIBNET_LINK, conf->iface, libnet_err);
        if (!lt) {
                dprint(LOG_ERR, "Unable to initialize libnet: %s",
                       libnet_err);
                global_error_exit("");
        }
        g.cleanup.libnet = lt;

        /* build headers */

        fw->type = 1;
        fw->next = 0;
        fw->ttl = conf->ttl;
        fw->table_id = conf->tid == INDEX_ANY ? 0 : conf->tid;

        /* calculate outgoing zFilter */
        /* read filter table from hw and set the filter accordingly */
        //        memset(outzf, 0, ZFILTER_SIZE);

        /* for now leave this unsupported and use user supplied zfilter */
        zf_str_to_hex((const char*)conf->zfilter, fw->zfilter);

        /* print packet that we are going to send out */
        dprint(LOG_DBG, "Payload:\n");
        dprint(LOG_DBG, "Header type: 0x%x\n", packet[0]);
        dprint(LOG_DBG, "Next header: 0x%x\n", packet[1]);
        dprint(LOG_DBG, "TTL: %d\n", packet[2]);
        dprint(LOG_DBG, "TID: %d\n", packet[3]);
        
        for (i = 0 ; i < ZFILTER_SIZE*2+32; i++) {
                dprint(LOG_DBG,"%02x ", packet[i]);
                if (i && !((i+1)%8))
                        dprint(LOG_DBG, " ");
                if (i && !((i+1)%16))
                        dprint(LOG_DBG, "\n");
        }
        
        /* read statistics */
        memset(&newcmd, 0, sizeof(newcmd));
        newcmd.command = CMD_GET | TGT_STATS;
        newcmd.prebuffer = malloc(sizeof(int) * 6 * g.nfconf.num_ports);
        if (!newcmd.prebuffer)
                global_error_exit("No memory");
                
        g.cleanup.extrabuf1 = newcmd.prebuffer;
        newcmd.port = INDEX_ANY;

        /* recursively call execute */
        execute(&newcmd);

        /* if we return here, then there was no error and we got the
           stats */

        for(i=0;i<conf->cnt;i++) {
                send_one_packet(lt, conf->proto, packet, conf->len);
                if (conf->delay > 0)
                        usleep(conf->delay*1000);
        }
        

        /* read statistics */

        memset(&newcmd, 0, sizeof(newcmd));
        newcmd.command = CMD_GET | TGT_STATS;
        newcmd.prebuffer = malloc(sizeof(int) * 6 * g.nfconf.num_ports);
        if (!newcmd.prebuffer)
                global_error_exit("No memory");
                
        g.cleanup.extrabuf2 = newcmd.prebuffer;
        newcmd.port = INDEX_ANY;

        /* recursively call execute */
        execute(&newcmd);

        /* compare against previous statistics */
        // new = new - old;
        
        new = g.cleanup.extrabuf2;
        old = g.cleanup.extrabuf1;
        for(i=0;i<g.nfconf.num_ports;i++)
                for(j=0;j<6;j++)
                        new[6*i+j] -= old[6*i+j];

	/*
        printf("Port:");
	for (i=0; i<g.nfconf.num_ports;i++)
		printf("\t%d", i);
	*/
	
	printf("Sent:");
	for (i=0; i<g.nfconf.num_ports;i++)
                printf("\t%d", new[i*6] == 0 ? 0 : 1);
        
        printf("\n");
        
        dprint(LOG_INF, "Port:\tOld:\tNew:\tString:\n");
        for (i = 0; i < g.nfconf.num_ports; i++) {
                dprint(LOG_INF, "%d\t%d\t%d\tspackets\n", i, old[6*i],
                       old[6*i]+new[6*i]);
                dprint(LOG_INF, "%d\t%d\t%d\tsbytes\n", i, old[6*i+1],
                       old[6*i+1]+new[6*i+1]);
                dprint(LOG_INF, "%d\t%d\t%d\trpackets\n", i, old[6*i+2],
                       old[6*i+2]+new[6*i+2]);
                dprint(LOG_INF, "%d\t%d\t%d\trbytes\n", i, old[6*i+3],
                       old[6*i+3]+new[6*i+3]);
                dprint(LOG_INF, "%d\t%d\t%d\tdropped1\n", i, old[6*i+4],
                       old[6*i+4]+new[6*i+4]);
                dprint(LOG_INF, "%d\t%d\t%d\tdropped2\n", i, old[6*i+5],
                       old[6*i+5]+new[6*i+5]);
        }

        /* g.cleanup_extracmd{1,2} contain before and after states */
        free(g.cleanup.extrabuf1);
        free(g.cleanup.extrabuf2);
        g.cleanup.extrabuf1 = NULL;
        g.cleanup.extrabuf2 = NULL;

        return;
}
