#ifndef _PSM_H
#define _PSM_H

#include <libnet.h>

/*** GLOBAL HARDWARE SPECIFIC VARIABLES ***/
#define INDEX_ANY (-1)
#define ZFILTER_SIZE 32
#define MAX_FILTER_FILESIZE 25000         /* bytes */


#define IN_VALID_PORT_RANGE(port) ((port) >= 0 &&                 \
                                   (port) < g.nfconf.num_ports)
#define IN_VALID_TID_RANGE(tid)   ((tid) >= 0 &&                  \
                                   (tid)  < g.nfconf.num_tids)
#define IN_VALID_LINK_RANGE(link) ((link) >= 0 &&                 \
                                   (link) < g.nfconf.num_links)

#define IS_IN_ZF_ZONE(addr)       ((addr) >= ZF_FILTER_BASE &&    \
                                   (addr) < ZF_CONFIG_BASE)

#define ZF_FILTER_TID(addr)       (((addr) & g.nfconf.tid_mask)   \
                                   >> ZF_FILTER_TID_OFF)

/* PL offset depends on variable length tid size */
#define ZF_FILTER_PL(addr)        (((addr) & g.nfconf.pl_mask)    \
                                   >> g.nfconf.pl_off)


#define SIMULATE                  (g.opt.simulate == 1)

/* simple macro to advance to the next zfilter */
#define NEXTFILTER(t,p,l,err) do {   \
    l++;                             \
    if (l >= g.nfconf.num_links) {   \
         l = 0; p++;                 \
    }                                \
    if (p >= g.nfconf.num_ports) {   \
         p = 0; t++;                 \
    }                                \
    if (t >= g.nfconf.num_tids)  {   \
         err = 1;                    \
    } else {                         \
         err = 0;                    \
    }                                \
} while(0)

/* z is of type zfilter (uint8_t *) (defined in configparser.h) */
#define GETFILTER(z,t,p,l) (z[t*g.nfconf.num_ports*g.nfconf.num_links+  \
                              p*g.nfconf.num_links+l])


typedef enum {
        CMD_NONE   = 0x0000,
        CMD_GET    = 0x0001,
        CMD_SET    = 0x0002,
        CMD_CLEAR  = 0x0003,
        CMD_SEND   = 0x0004,
        CMD_MASK   = 0x00FF,
        TGT_FILTER = 0x0100,
        TGT_STATS  = 0x0200,
        TGT_PACKET = 0x0300,
        TGT_MASK   = 0x0F00
} cmd_t;

#define GET_CMD(psm) (((psm)->command) & CMD_MASK)
#define GET_TGT(psm) (((psm)->command) & TGT_MASK)
#define IS_TGT_SET(psm, tgt) (GET_TGT(psm) == tgt)

#define LOG_ERR  0
#define LOG_INF  1
#define LOG_DBG  2

struct psm_conf {
        cmd_t command;
        int port;
        int tid;
        int link;
        uint8_t zfilter[ZFILTER_SIZE*2]; /* two characters per one byte */
        FILE *file;
        char *prebuffer;
        int ttl;
        char *iface;
        int cnt;
        int delay;
        int len;
        int proto;
};


#define NF_READZF  1   /* ZFilter read */
#define NF_WRITEZF 2  
#define NF_READ    3  /* due to hardware issues separate reads for zfilters */
#define NF_WRITE   4  

struct nf2_cmd {
        int cmd;
        int addr;
        int len;
        int tag;
        uint8_t *target;
        struct nf2_cmd *next;
        struct nf2_cmd *first;
};


struct global_args {
        struct {
                char *buffer;
                int toparse;
                int offset;
                int mode;
                FILE *file;
        } parser;  /* parser related globals */
        struct {
                int verbose;
                char *nf2_dev;
                int simulate;
        } opt;   /* command line related variables */
        struct {
                int num_ports;
                int num_tids;
                int num_links;
                uint32_t tid_mask;
                uint32_t pl_mask;
                int pl_off;
        } nfconf; /* queried configuration of nf2 zfilter implementation */
        struct {
                struct nf2_cmd *cmdchain;
                uint8_t *packet;
                libnet_t *libnet;
                void *extrabuf1;
                void *extrabuf2;
                void *iface;
        } cleanup;
        struct nf2handle *nf2;
        ztable rttable;
};


struct zfilter_fwd_header {
        uint8_t type;
        uint8_t next;
        uint8_t ttl;
        uint8_t table_id;
        uint8_t reserved[6];
        uint8_t zfilter[ZFILTER_SIZE];
} __attribute__ ((__packed__));

void usage();
int parse_args_opts(int argc, char **argv);
int parse_args(int argc, char **argv, int off, struct psm_conf *psm);
void execute(struct psm_conf *conf);


#define GCC_VERSION ( __GNUC__ * 10000 + __GNUC_MINOR__ * 100 + \
                      __GNUC_PATCHLEVEL__)


#if GCC_VERSION >= 40302
/* this is workaround for compiler (probably due to libnet not compiling
 * cleanly (=without warnings) when compiled with -std=gnu99).
 */
inline uint32_t libnet_getgre_length(uint16_t x)
{
  return 0;
}
#endif


#endif
