#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>

#include "../common/reg_defines.h"
#include "../common/nf2.h"
#include "../common/nf2util.h"

#include "../PubSubManager/configparser.h"

#define DEFAULT_IFACE "nf2c0"

static struct nf2device nf2;
int debug = 0;

void usage(char * name)
{
	fprintf(stderr, "Usage: %s [options] <Link ID file>\n"
		"-h               Print this help\n"
		"-d <dbg level>   Debug level (default: 0)\n"
		"-i <interface>   Interface to use for configuration "
		"(default: nf2c0)\n", name);
}


int parse_args(int argc, char *argv[])
{
	char c;
	FILE * f = NULL;

	if (argc <= 1 || argv[argc-1][0] == '-') {
		fprintf(stderr, "Link ID file not given\n");
		usage(argv[0]);
		return EXIT_FAILURE;
	}
	if ((f = fopen(argv[argc-1], "r")) == NULL) {
		fprintf(stderr, "Failed to open link ID file '%s'\n",
			argv[argc-1]);
		usage(argv[0]);
		return EXIT_FAILURE;
	}
	fclose(f);

	opterr = 0;
	while ((c = getopt(argc, argv, "hd:i:")) != -1) {
		switch (c) {
			case 'h':
				usage(argv[0]);
				return EXIT_FAILURE;
			case 'd':
				debug = atoi(optarg);
				break;
			case 'i':
				nf2.device_name = optarg;
				break;
			case '?':
				fprintf(stderr, "Unknown option '-%c'.\n",
				        optopt);
			default:
				usage(argv[0]);
				return EXIT_FAILURE;
		}
	}
	return EXIT_SUCCESS;
}

int main(int argc, char *argv[])
{
	port filter = NULL;
	int i, j, k;
	uint8_t *data;

	nf2.device_name = DEFAULT_IFACE;

	if (parse_args(argc, argv)) {
		return EXIT_FAILURE;
	}

	if (!(filter = cp_read_and_parse((const char *)argv[argc - 1]))) {
		fprintf(stderr, "Failed to parse link IDs from file: '%s'\n",
		        argv[argc - 1]);
		usage(argv[0]);
		return EXIT_FAILURE;
	}
	
	for (i = 0; i < MAXPORT + 1; i++) {
		printf("Port %d:\n", i);
		for (j = 0; j < MAXLID + 1; j++) {
			printf("\tLID %d: ", j);
			data = filter[i][j];
			if (data == NULL)
				printf("(null)");
			else
				for(k = 0; k < LID_STR_LEN / 2; k++)
					printf("%02x", data[k]);
			printf("\n");

		}
	}
	return EXIT_SUCCESS;
}

