#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <net/if.h>
//#include <linux/if.h>
#include <arpa/inet.h>

#include "api.h"

/* See api.h for documentation */

struct nf2handle *nflib_open(const char *device_name)
{
        const char *name = NFLIB_DEF_IFACE_NAME;
        struct ifreq ifreq;                           
        int sock;
        struct nf2handle *nf2;

        if (device_name)
                name = device_name;

        if (geteuid() != 0)
                return NULL;


        nf2 = malloc(sizeof(*nf2));
        if (!nf2)
                return NULL;
        sock = socket(AF_INET, SOCK_DGRAM, 0);
        if (sock == -1)
                return NULL;

        memset(&ifreq, 0, sizeof(ifreq));
        strncpy(ifreq.ifr_ifrn.ifrn_name, name, IFNAMSIZ);
        if (setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE, (char *)&ifreq,      
                       sizeof(ifreq)) < 0)
                return NULL;

        nf2->sock = sock;
        strncpy(nf2->name, name, IFNAMSIZ);

        return nf2;
}

void nflib_close(struct nf2handle *nf2)
{
        close(nf2->sock);
        free(nf2);
}

int nflib_readreg(struct nf2handle *nf2, uint32_t reg, uint32_t *value)
{
        int ret;
        struct ifreq ifreq = {0};
        struct nf2reg nf2reg = {0};

        nf2reg.reg = reg;

        ifreq.ifr_data = (char *)&nf2reg;
        strncpy(ifreq.ifr_ifrn.ifrn_name, nf2->name, IFNAMSIZ);
                
        ret = ioctl(nf2->sock, SIOCREGREAD, &ifreq);
        if (ret == -1)
                perror("Error reading register");
        else
                *value = nf2reg.val;

        return ret;
}

int nflib_read(struct nf2handle *nf2, uint32_t reg, uint8_t *data, int size)
{
        int i;
        uint32_t *data32;
        int err;

        /* Currently all multiple reads must be aligned */
        if (size & (NFLIB_ALIGN32 - 1))
                return -EFAULT;

        data32 = (uint32_t *)data;

        for(i=0; i<size; i+=4) {
                err = nflib_readreg(nf2, reg++, data32++);
                if (err)
                        return err;
        }

        return 0;
}

int nflib_writereg(struct nf2handle *nf2, uint32_t reg, uint32_t value)
{
        int ret;
        struct nf2reg nf2reg;
        struct ifreq ifreq = {0};

        nf2reg.reg = reg;
        nf2reg.val = value;
               
        ifreq.ifr_data = (char *)&nf2reg;
        strncpy(ifreq.ifr_ifrn.ifrn_name, nf2->name, IFNAMSIZ);

        ret = ioctl(nf2->sock, SIOCREGWRITE, &ifreq);
        return ret;
}

int nflib_write(struct nf2handle *nf2, uint32_t reg, uint8_t *data, int size)
{
        int i;
        uint32_t *data32;
        int err;

        /* Currently all multiple reads must be aligned */
        if (size & (NFLIB_ALIGN32 - 1))
                return -EFAULT;
       
        data32 = (uint32_t *)data;

        for(i=0; i<size; i+=4) {
                err = nflib_writereg(nf2, reg++, *data32++);
                if (err)
                        return err;
        }

        return 0;
}

