#ifndef _NFLIB_API_H_
#define _NFLIB_API_H_

#define NFLIB_DEF_IFACE_NAME   "nf2c0"
#define NFLIB_ALIGN32          4
#define NFLIB_REG_ALIGN(x)     (x & ~(NFLIB_ALIGN32-1))

#define SIOCREGREAD            SIOCDEVPRIVATE
#define SIOCREGWRITE           (SIOCDEVPRIVATE + 1)

/*
 * Structure for transferring register data via an IOCTL
 */
struct nf2reg {
        unsigned int    reg;
        unsigned int    val;
};

struct nf2handle {
        int      sock;
        char     name[IFNAMSIZ];
};

/** nflib_open - Open connection to NetFPGA device using socket
 *
 * device_name = Name of NetFPGA user space network interface
 *
 * Returns file descriptor (socket) that must be closed() on exit,
 * or an error code:
 *
 * -EPERM = Not super user
 * -ENOTAVAIL = Could not allocate socket
 * -ENOLINK = Could not bind socket to device
 *
 * By default "nf2c0" is used as network interface name
 */
struct nf2handle *nflib_open(const char *device_name);

void nflib_close(struct nf2handle *nf2);

/**
 * nflib_readreg  - Read a 32-bit register
 *
 * fd - file descriptor to the device
 * reg - address of the register
 * value - pointer to where the register content is stored
 *
 * Returns: Check "man ioctl" for return values
 *
 */
int nflib_readreg(struct nf2handle *fd, uint32_t reg, uint32_t *value);


/**
 * nflib_read  - Read bytes from register
 *
 * fd - file descriptor to the device
 * reg - address of the register
 * data - pointer to where the register content is stored
 * size - amount of bytes to read (must be 32-bit aligned)
 *
 * Returns: Check "man ioctl" for return values
 *
 */
int nflib_read(struct nf2handle *fd, uint32_t reg, uint8_t *data, int size);


/**
 * nflib_writereg  - Write a 32-bit register
 *
 * fd - file descriptor to the device
 * reg - address of the register
 * value - content to be written to the register
 *
 * Returns: Check "man ioctl" for return values
 *
 */
int nflib_writereg(struct nf2handle *fd, uint32_t reg, uint32_t value);


/**
 * nflib_write  - Write bytes to register
 *
 * fd - file descriptor to the device
 * reg - address of the register
 * data - content to be written to the register
 * size - amount of bytes to write (must be 32-bit aligned)
 *
 * Returns: Check "man ioctl" for return values
 *
 */
int nflib_write(struct nf2handle *fd, uint32_t reg, uint8_t *data, int size);

#endif
